# usage: print_usage_text SINK
# SINK is 1 for stdout or 2 for stderr
function print_usage_text {
cat <<END >&$1
$SCRIPTNAME - MIRROR Release Including Backports, for Debian
general usage: $SCRIPTNAME [OPTIONS] DEBIAN-VER ARGUMENTS
usage for buster:
END
if (($ENABLE_EXPERIMENTAL)); then
cat <<END >&$1
  $SCRIPTNAME [-lsiIBt?-] [-p FILE] 10.N CD-REV ARCH OLD TARGET ISO MIRROR [LANGS...]
END
else
cat <<END >&$1
  $SCRIPTNAME [-ls?-] [-p FILE] 10.N CD-REV ARCH OLD TARGET ISO MIRROR [LANGS...]
END
fi
cat <<END >&$1
usage for bullseye: (not yet implemented)
OPTIONS
    -l save local disk space by hard-linking files from OLD to TARGET
    -s change nothing, but print checksums from control
END
(($ENABLE_EXPERIMENTAL)) && cat <<END >&$1
    -i (experimental) fetch only iso cd boot files
    -I (experimental) skip iso cd boot files
    -B (experimental) omit backports
    -t (experimental) fetch only a test sample
END
cat <<END >&$1
    -? print this help message
    -- disable further option processing
    -p FILE stamp the named file with the referenced snapshot's date and time
DEBIAN-VER debian version,  e.g. 10.8
CD-REV     iso cd revision, e.g. 0 (for cd version 10.8.0)
ARCH       architecture,    e.g. amd64
OLD        old target dir,  e.g. ~/debian-10.7 (enter /dev/null if none)
TARGET     new target dir,  e.g. ~/debian-10.8
ISO        iso target dir,  e.g. ~/debian-cd-10.8.0
MIRROR     mirror,          e.g. mirror.example.org
LANGS      languages,       e.g. en fr de de_DE (enter @ for all languages)
Debian's snapshot service has limited capacity and must therefore ban abusers;
but you can mitigate the load on snapshot.debian.org (and also on MIRROR) by
specifying a former TARGET, rather than /dev/null, as OLD. For MIRROR, you may
choose from the list at [https://www.debian.org/mirror/] a single mirror that
serves both debian/ and debian-cd/.
END
}

# Read and process the command line.
#
# (The program begins to grow a little too large for the
# following, OPT-controlled architecture to be entirely comfortable.
# The architecture might want redesign at some future date.)
OPT_HARDLINK=0
OPT_SUMS=0
OPT_PHASE_A=1   # check mirror availability
OPT_PHASE_B=1   # determine the backports date
OPT_PHASE_C=1   # fetch dist-root control
OPT_PHASE_D=1   # fetch per-area control
OPT_PHASE_DD=1  # stamp the release time
OPT_PHASE_E=1   # fetch pool files
OPT_PHASE_F=1   # fetch iso cd boot files
OPT_PHASE_G=1   # set permissions and stamp directories
OPT_BACKPORTS=1 # fetch backports (provided that other options allow it)
OPT_SMALL_TEST=0
OPT_STAMP=0     # return the snapshot's date via a timestamp
STAMPOPT_FILE=''
AVAILABLE_OPTS=':ls-p:';\
 (($ENABLE_EXPERIMENTAL)) && AVAILABLE_OPTS=':lsiIBt-p:';
CONTINUE_PROCESSING_OPTS=1; LAST_OPTIND=$OPTIND
while getopts "$AVAILABLE_OPTS" OPTLETTER; do
    case "$OPTLETTER" in
        l) OPT_HARDLINK=1;;
        s) OPT_SUMS=1; OPT_PHASE_A=0; OPT_PHASE_B=0; OPT_PHASE_C=0;\
            OPT_PHASE_DD=0; OPT_PHASE_F=0; OPT_PHASE_G=0;;
        i) OPT_PHASE_B=0; OPT_PHASE_C=0; OPT_PHASE_D=0;\
            OPT_PHASE_DD=0; OPT_PHASE_E=0;;
        I) OPT_PHASE_F=0;;
        B) OPT_BACKPORTS=0; OPT_PHASE_B=0;;
        t) OPT_SMALL_TEST=1;;
        -) CONTINUE_PROCESSING_OPTS=0;;
        p) OPT_STAMP=1; STAMPOPT_FILE="$OPTARG";;
        *)
            SINK=1; [ "$OPTARG" = '?' ] || SINK=2
            print_usage_text "$SINK"
            [ "$OPTARG" = '?' ] && exit 0; exit 1
        ;;
    esac
    (($CONTINUE_PROCESSING_OPTS || $LAST_OPTIND == $OPTIND)) || break;
    LAST_OPTIND=$OPTIND
done
shift $(($OPTIND - 1))
(($OPT_STAMP && $OPT_SUMS)) && die "unable to execute the -p option"\
 "because, when the -s option is in effect, the program does not"\
 "connect to the snapshot service"
(($OPT_HARDLINK && $OPT_SUMS)) && warn "the -l option does nothing"\
 "while the -s option is in effect"
[ $# -ge 7 ] || { print_usage_text 2; exit 1; }
die_if_zsp "$@"
require_filenames_to_be_legal "$1" "$2" "$3" "$7"
require_filepaths_to_be_legal "$4" "$5" "$6"
VER="$1"; ISOCD_VER="$1.$2"; ARCH="$3"; MIRROR_PRIMARY="$7"
OLD="$(realpath -e "$4")"
TARGET="$(realpath -m "$5")";
ISOCD_TARGET="$(realpath -m "$6")"
die_if_zsp              "$OLD" "$TARGET" "$ISOCD_TARGET"
require_filepaths_to_be_legal "$OLD" "$TARGET" "$ISOCD_TARGET"
shift 7
for LANG1 in "$@"; do
    [ "$LANG1" = '@' ] || require_filenames_to_be_legal "$LANG1"
done
(($OPT_PHASE_C)) && [ "$TARGET" = "$OLD" ] && {
    die "$TARGET must not be both OLD and TARGET"
}
(($OPT_PHASE_F)) && [ "$ISOCD_TARGET" = "$OLD" ] && {
    die "$ISOCD_TARGET must not be both OLD and ISO"
}
(($OPT_PHASE_C && $OPT_PHASE_F)) && [ "$ISOCD_TARGET" = "$TARGET" ] && {
    die "ISO must be distinct from TARGET"
}
(($OPT_PHASE_C)) && [ -e "$TARGET" ] && {
    die "$TARGET already exists;"\
    "if you do not want it, then delete it before invoking this program"
}
(($OPT_PHASE_F)) && [ -e "$ISOCD_TARGET" ] && {
    die "$ISOCD_TARGET already exists;"\
    "if you do not want it, then delete it before invoking this program"
}
LANGS=("$@")
MAJOR_VER="${VER%%.*}"
MINOR_VER="${VER#*.}"
INSTALLER_VER="deb${MAJOR_VER}u${MINOR_VER}"
case "$MAJOR_VER" in
    10)
        DIST_CODENAME="$DIST10"
        INSTALLER_R0_DATE="$INSTALLER_R0_DATE10"
    ;;
    # Future major versions of Debian can be added here.
    *) die "Debian version $MAJOR_VER is not supported";;
esac
DIST="$PROJECT_NAME$VER"; (($OPT_SUMS)) && DIST="$DIST_CODENAME"
DIST_BACKPORTS="$DIST_CODENAME-backports"

